//+------------------------------------------------------------------+
//|                                                           VA.mq4 |
//|                               Copyright  2013, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Green
#property indicator_color2 Red

extern string  TimeFrame       = "Current time frame";
extern int Velocity_Length     = 14;
extern int Acceleration_Length = 10;
extern int Price               = 0;      // Applied price
                                         // 0 - Close
                                         // 1 - Open
                                         // 2 - High
                                         // 3 - Low
                                         // 4 - Median
                                         // 5 - Typical
                                         // 6 - Weighted  
extern double smooth           = 5.5;

double V[], A[];
int MaxLength;
string indicatorFileName;
bool   returnBars;
int    timeFrame;

int init()
{
 IndicatorShortName("Velocity/Acceleration");
 IndicatorDigits(Digits);
 SetIndexStyle(0,DRAW_LINE);
 SetIndexBuffer(0,V);
 SetIndexStyle(1,DRAW_LINE);
 SetIndexBuffer(1,A);
 MaxLength=MathMax(Velocity_Length, Acceleration_Length);
 indicatorFileName = WindowExpertName();
 returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
 timeFrame         = stringToTimeFrame(TimeFrame);
 IndicatorShortName(timeFrameToString(timeFrame)+"  VA smoothed");
 return(0);
}

int deinit()
{

 return(0);
}

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { V[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            for (int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  V[i] = iCustom(NULL,timeFrame,indicatorFileName,"",Velocity_Length,Acceleration_Length,Price,smooth,0,y);
                  A[i] = iCustom(NULL,timeFrame,indicatorFileName,"",Velocity_Length,Acceleration_Length,Price,smooth,1,y);
            }
            return(0);                
         }
 if(Bars<=MaxLength) return(0);
 int ExtCountedBars=IndicatorCounted();
 if (ExtCountedBars<0) return(-1);
 limit=Bars-MaxLength;
 if(ExtCountedBars>2) limit=Bars-ExtCountedBars-1;
 int pos;
 double Pr0, PrL;
 pos=limit;
 while(pos>=0)
 {
  Pr0=iMA(NULL, 0, 1, 0, MODE_SMA, Price, pos);
  PrL=iMA(NULL, 0, 1, 0, MODE_SMA, Price, pos+Velocity_Length);
  if (PrL!=0)
  {
   V[pos]=iSsm(100*Pr0/PrL,smooth,pos,0);
  } 
  pos--;
 } 
 
 pos=limit;
 while(pos>=0)
 {
  if (V[pos+Acceleration_Length]!=0)
  {
      A[pos]=iSsm(100*V[pos]/V[pos+Acceleration_Length],smooth,pos,1);
  } 
  pos--;
 }  
 return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSsm[][4];
#define _price  0
#define _ssm    1

double workSsmCoeffs[][4];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define Pi 3.14159265358979323846264338327950288

//
//
//
//
//

double iSsm(double price, double period, int i, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSsm,0) !=Bars)                 ArrayResize(workSsm,Bars);
   if (ArrayRange(workSsmCoeffs,0) < (instanceNo+1)) ArrayResize(workSsmCoeffs,instanceNo+1);
   if (workSsmCoeffs[instanceNo][_period] != period)
   {
      workSsmCoeffs[instanceNo][_period] = period;
      double a1 = MathExp(-1.414*Pi/period);
      double b1 = 2.0*a1*MathCos(1.414*Pi/period);
         workSsmCoeffs[instanceNo][_c2] = b1;
         workSsmCoeffs[instanceNo][_c3] = -a1*a1;
         workSsmCoeffs[instanceNo][_c1] = 1.0 - workSsmCoeffs[instanceNo][_c2] - workSsmCoeffs[instanceNo][_c3];
   }

   //
   //
   //
   //
   //

      i = Bars-i-1;
      int s = instanceNo*2;   
          workSsm[i][s+_price] = price;
          workSsm[i][s+_ssm]   = workSsmCoeffs[instanceNo][_c1]*(workSsm[i][s+_price]+workSsm[i-1][s+_price])/2.0 + 
                                 workSsmCoeffs[instanceNo][_c2]*workSsm[i-1][s+_ssm]                              + 
                                 workSsmCoeffs[instanceNo][_c3]*workSsm[i-2][s+_ssm];
   return(workSsm[i][s+_ssm]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


